@@ -63,6 +63,80 @@ namespace thrust
  *  \{
  */
 
+namespace detail
+{
+  
+template <typename T, std::size_t Align>
+struct complex_storage;
+
+#if THRUST_CPP_DIALECT >= 2011                                                    \
+  && (THRUST_HOST_COMPILER == THRUST_HOST_COMPILER_GCC)                       \
+  && (THRUST_GCC_VERSION >= 40800)
+  // C++11 implementation, excluding GCC 4.7, which doesn't have `alignas`.
+  template <typename T, std::size_t Align>
+  struct complex_storage
+  {
+    struct alignas(Align) type { T x; T y; };
+  };
+#elif  (THRUST_HOST_COMPILER == THRUST_HOST_COMPILER_MSVC)                    \
+    || (   (THRUST_HOST_COMPILER == THRUST_HOST_COMPILER_GCC)                 \
+        && (THRUST_GCC_VERSION < 40600))
+  // C++03 implementation for MSVC and GCC <= 4.5.
+  // 
+  // We have to implement `aligned_type` with specializations for MSVC
+  // and GCC 4.2 and older because they require literals as arguments to 
+  // their alignment attribute.
+
+  #if (THRUST_HOST_COMPILER == THRUST_HOST_COMPILER_MSVC)
+    // MSVC implementation.
+    #define THRUST_DEFINE_COMPLEX_STORAGE_SPECIALIZATION(X)                   \
+      template <typename T>                                                   \
+      struct complex_storage<T, X>                                            \
+      {                                                                       \
+        __declspec(align(X)) struct type { T x; T y; };                       \
+      };                                                                      \
+      /**/
+  #else
+    // GCC <= 4.2 implementation.
+    #define THRUST_DEFINE_COMPLEX_STORAGE_SPECIALIZATION(X)                   \
+      template <typename T>                                                   \
+      struct complex_storage<T, X>                                            \
+      {                                                                       \
+        struct type { T x; T y; } __attribute__((aligned(X)));                \
+      };                                                                      \
+      /**/
+  #endif
+
+  // The primary template is a fallback, which doesn't specify any alignment.
+  // It's only used when T is very large and we're using an older compilers
+  // which we have to fully specialize each alignment case.
+  template <typename T, std::size_t Align>
+  struct complex_storage
+  {
+    T x; T y;
+  };
+  
+  THRUST_DEFINE_COMPLEX_STORAGE_SPECIALIZATION(1);
+  THRUST_DEFINE_COMPLEX_STORAGE_SPECIALIZATION(2);
+  THRUST_DEFINE_COMPLEX_STORAGE_SPECIALIZATION(4);
+  THRUST_DEFINE_COMPLEX_STORAGE_SPECIALIZATION(8);
+  THRUST_DEFINE_COMPLEX_STORAGE_SPECIALIZATION(16);
+  THRUST_DEFINE_COMPLEX_STORAGE_SPECIALIZATION(32);
+  THRUST_DEFINE_COMPLEX_STORAGE_SPECIALIZATION(64);
+  THRUST_DEFINE_COMPLEX_STORAGE_SPECIALIZATION(128);
+
+  #undef THRUST_DEFINE_COMPLEX_STORAGE_SPECIALIZATION
+#else
+  // C++03 implementation for GCC > 4.5, Clang, PGI, ICPC, and xlC.
+  template <typename T, std::size_t Align>
+  struct complex_storage
+  {
+    struct type { T x; T y; } __attribute__((aligned(Align)));
+  };
+#endif
+
+} // end namespace detail
+
   /*! \p complex is the Thrust equivalent to <tt>std::complex</tt>. It is
    *  functionally identical to it, but can also be used in device code which
    *  <tt>std::complex</tt> currently cannot.
@@ -84,11 +158,6 @@ public:
 
   /* --- Constructors --- */
 
-  /*! Default construct a complex number.
-   */
-  __host__ __device__
-  complex();
-
   /*! Construct a complex number with an imaginary part of 0.
    *
    *  \param re The real part of the number.
@@ -104,6 +173,23 @@ public:
   __host__ __device__
   complex(const T& re, const T& im);
 
+#if THRUST_CPP_DIALECT >= 2011
+  /*! Default construct a complex number.
+   */
+  complex() = default;
+
+  /*! This copy constructor copies from a \p complex with a type that is
+   *  convertible to this \p complex's \c value_type.
+   *
+   *  \param z The \p complex to copy from.
+   */
+  complex(const complex<T>& z) = default;
+#else
+  /*! Default construct a complex number.
+   */
+  __host__ __device__
+  complex();
+
   /*! This copy constructor copies from a \p complex with a type that is
    *  convertible to this \p complex's \c value_type.
    *
@@ -111,6 +197,7 @@ public:
    */
   __host__ __device__
   complex(const complex<T>& z);
+#endif
 
   /*! This converting copy constructor copies from a \p complex with a type
    *  that is convertible to this \p complex's \c value_type.
@@ -130,7 +217,7 @@ public:
    */
   __host__ THRUST_STD_COMPLEX_DEVICE
   complex(const std::complex<T>& z);
-  
+
   /*! This converting copy constructor copies from a <tt>std::complex</tt> with
    *  a type that is convertible to this \p complex's \c value_type.
    *
@@ -154,6 +241,14 @@ public:
   __host__ __device__
   complex& operator=(const T& re);
 
+#if THRUST_CPP_DIALECT >= 2011
+  /*! Assign `z.real()` and `z.imag()` to the real and imaginary parts of this
+   *  \p complex respectively.
+   *
+   *  \param z The \p complex to copy from.
+   */
+  complex& operator=(const complex<T>& z) = default;
+#else
   /*! Assign `z.real()` and `z.imag()` to the real and imaginary parts of this
    *  \p complex respectively.
    *
@@ -161,6 +256,7 @@ public:
    */
   __host__ __device__
   complex& operator=(const complex<T>& z);
+#endif
 
   /*! Assign `z.real()` and `z.imag()` to the real and imaginary parts of this
    *  \p complex respectively.
@@ -180,7 +276,7 @@ public:
    */
   __host__ THRUST_STD_COMPLEX_DEVICE
   complex& operator=(const std::complex<T>& z);
-  
+
   /*! Assign `z.real()` and `z.imag()` to the real and imaginary parts of this
    *  \p complex respectively.
    *
@@ -193,7 +289,6 @@ public:
   complex& operator=(const std::complex<U>& z);
 
 
-
   /* --- Compound Assignment Operators --- */
 
   /*! Adds a \p complex to this \p complex and assigns the result to this
@@ -356,27 +451,7 @@ public:
   operator std::complex<T>() const { return std::complex<T>(real(), imag()); }
 
 private:
-  struct generic_storage_type { T x; T y; };
-
-#if THRUST_DEVICE_COMPILER == THRUST_DEVICE_COMPILER_NVCC
-  typedef typename detail::conditional<
-    detail::is_same<T, float>::value, float2,
-    typename detail::conditional<
-      detail::is_same<T, float const>::value, float2 const,
-      typename detail::conditional<
-        detail::is_same<T, double>::value, double2,
-        typename detail::conditional<
-          detail::is_same<T, double const>::value, double2 const,
-          generic_storage_type
-        >::type
-      >::type
-    >::type
-  >::type storage_type;
-#else
-  typedef generic_storage_type storage_type;
-#endif
-
-  storage_type data;
+  typename detail::complex_storage<T, sizeof(T) * 2>::type data;
 };
 
 
