@@ -24,7 +24,7 @@
 
 #define THRUST_MR_DEFAULT_ALIGNMENT THRUST_ALIGNOF(::thrust::detail::max_align_t)
 
-#if __cplusplus >= 201703L
+#if THRUST_CPP_DIALECT >= 2017
 #  if __has_include(<memory_resource>)
 #    define THRUST_MR_STD_MR_HEADER <memory_resource>
 #    define THRUST_MR_STD_MR_NS std::pmr
