@@ -21,10 +21,11 @@
 
 #pragma once
 
-#include <algorithm>
+#include <thrust/detail/algorithm_wrapper.h>
 
 #include <thrust/host_vector.h>
 #include <thrust/binary_search.h>
+#include <thrust/detail/seq.h>
 
 #include <thrust/mr/memory_resource.h>
 #include <thrust/mr/allocator.h>
@@ -37,10 +38,8 @@ namespace thrust
 namespace mr
 {
 
-/*! \addtogroup memory_management Memory Management
- *  \addtogroup memory_management_classes Memory Management Classes
- *  \addtogroup memory_resources Memory Resources
- *  \ingroup memory_resources
+/** \addtogroup memory_resources Memory Resources
+ *  \ingroup memory_management_classes
  *  \{
  */
 
@@ -114,8 +113,8 @@ public:
         m_smallest_block_log2(detail::log2_ri(m_options.smallest_block_size)),
         m_pools(m_bookkeeper),
         m_allocated(m_bookkeeper),
-        m_oversized(m_bookkeeper),
-        m_cached_oversized(m_bookkeeper)
+        m_cached_oversized(m_bookkeeper),
+        m_oversized(m_bookkeeper)
     {
         assert(m_options.validate());
 
@@ -137,8 +136,8 @@ public:
         m_smallest_block_log2(detail::log2_ri(m_options.smallest_block_size)),
         m_pools(m_bookkeeper),
         m_allocated(m_bookkeeper),
-        m_oversized(m_bookkeeper),
-        m_cached_oversized(m_bookkeeper)
+        m_cached_oversized(m_bookkeeper),
+        m_oversized(m_bookkeeper)
     {
         assert(m_options.validate());
 
@@ -250,6 +249,10 @@ private:
         {
         }
 
+#if THRUST_CPP_DIALECT >= 2011
+        pool & operator=(const pool &) = default;
+#endif
+
         __host__
         ~pool() {}
 
