@@ -32,7 +32,8 @@
 #include <cassert>
 
 
-THRUST_BEGIN_NS
+namespace thrust
+{
 
 namespace cuda_cub {
 namespace launcher {
@@ -45,7 +46,7 @@ namespace launcher {
     Size const shared_mem;
     cudaStream_t const stream;
 
-    CUB_RUNTIME_FUNCTION
+    THRUST_RUNTIME_FUNCTION
     triple_chevron(dim3         grid_,
                    dim3         block_,
                    Size         shared_mem_ = 0,
@@ -834,10 +835,14 @@ namespace launcher {
     }
 
 
-#ifdef __CUDA_ARCH__
-#define THRUST_TRIPLE_LAUNCHER_HOSTDEVICE doit_device
+#if defined(__NVCOMPILER_CUDA__)
+#  define THRUST_TRIPLE_LAUNCHER_HOSTDEVICE(...) \
+      (__builtin_is_device_code() ?              \
+          doit_device(__VA_ARGS__) : doit_host(__VA_ARGS__))
+#elif defined(__CUDA_ARCH__)
+#  define THRUST_TRIPLE_LAUNCHER_HOSTDEVICE doit_device
 #else
-#define THRUST_TRIPLE_LAUNCHER_HOSTDEVICE doit_host
+#  define THRUST_TRIPLE_LAUNCHER_HOSTDEVICE doit_host
 #endif
 
 #if 0
@@ -968,4 +973,4 @@ namespace launcher {
 }    // namespace launcher
 }    // namespace cuda_
 
-THRUST_END_NS
+} // end namespace thrust
