@@ -32,37 +32,49 @@
 #include <thrust/type_traits/is_contiguous_iterator.h>
 #include <thrust/detail/raw_pointer_cast.h>
 #include <thrust/system/cuda/detail/util.h>
-#include <thrust/system/cuda/detail/cub/block/block_load.cuh>
-#include <thrust/system/cuda/detail/cub/block/block_store.cuh>
-#include <thrust/system/cuda/detail/cub/block/block_scan.cuh>
+#include <cub/block/block_load.cuh>
+#include <cub/block/block_store.cuh>
+#include <cub/block/block_scan.cuh>
 
-
-THRUST_BEGIN_NS
+namespace thrust
+{
 
 namespace cuda_cub {
 namespace core {
 
-#if (__CUDA_ARCH__ >= 600)
-#  define THRUST_TUNING_ARCH sm60
-#elif (__CUDA_ARCH__ >= 520)
-#  define THRUST_TUNING_ARCH sm52
-#elif (__CUDA_ARCH__ >= 350)
-#  define THRUST_TUNING_ARCH sm35
-#elif (__CUDA_ARCH__ >= 300)
-#  define THRUST_TUNING_ARCH sm30
-#elif !defined (__CUDA_ARCH__)
-#  define THRUST_TUNING_ARCH sm30
+#ifdef __NVCOMPILER_CUDA__
+#  if (__NVCOMPILER_CUDA_ARCH__ >= 600)
+#    define THRUST_TUNING_ARCH sm60
+#  elif (__NVCOMPILER_CUDA_ARCH__ >= 520)
+#    define THRUST_TUNING_ARCH sm52
+#  elif (__NVCOMPILER_CUDA_ARCH__ >= 350)
+#    define THRUST_TUNING_ARCH sm35
+#  else
+#    define THRUST_TUNING_ARCH sm30
+#  endif
+#else
+#  if (__CUDA_ARCH__ >= 600)
+#    define THRUST_TUNING_ARCH sm60
+#  elif (__CUDA_ARCH__ >= 520)
+#    define THRUST_TUNING_ARCH sm52
+#  elif (__CUDA_ARCH__ >= 350)
+#    define THRUST_TUNING_ARCH sm35
+#  elif (__CUDA_ARCH__ >= 300)
+#    define THRUST_TUNING_ARCH sm30
+#  elif !defined (__CUDA_ARCH__)
+#    define THRUST_TUNING_ARCH sm30
+#  endif
 #endif
 
   // Typelist - a container of types, supports up to 10 types
   // --------------------------------------------------------------------------
-  
+
   class _;
   template <class = _, class = _, class = _, class = _, class = _, class = _, class = _, class = _, class = _, class = _>
   struct typelist;
 
   // -------------------------------------
-  
+
   // supported SM arch
   // ---------------------
   struct sm30  { enum { ver = 300, warpSize = 32 }; };
@@ -94,7 +106,7 @@ namespace core {
 
   // metafunction to match next viable PtxPlan specialization
   // --------------------------------------------------------------------------
- 
+
   __THRUST_DEFINE_HAS_NESTED_TYPE(has_tuning_t, tuning)
   __THRUST_DEFINE_HAS_NESTED_TYPE(has_type_t, type)
 
@@ -121,7 +133,7 @@ namespace core {
             template <class, class> class Tuning,
             class _0>
   struct has_sm_tuning_impl<SM, Tuning<lowest_supported_sm_arch, _0> > : has_type_t<Tuning<SM, _0> > {};
-  
+
   // specializing for Tunig which needs 2 args
   template <class SM,
             template <class, class,class> class Tuning,
@@ -131,9 +143,9 @@ namespace core {
   template <template <class> class P, class SM>
   struct has_sm_tuning : has_sm_tuning_impl<SM, typename P<lowest_supported_sm_arch>::tuning > {};
 
-  // once first match is found in sm_list, all remaining sm are possible 
+  // once first match is found in sm_list, all remaining sm are possible
   // candidate for tuning, so pick the first available
-  //   if the plan P has SM-level tuning then pick it, 
+  //   if the plan P has SM-level tuning then pick it,
   //   otherwise move on to the next sm in the sm_list
   template <template <class> class P, class SM, class _1, class _2, class _3, class _4, class _5, class _6, class _7, class _8, class _9>
   struct specialize_plan_impl_match<P, typelist<SM, _1, _2, _3, _4, _5, _6, _7, _8, _9> >
@@ -146,14 +158,14 @@ namespace core {
     struct specialize_plan_msvc10_war
     {
       // if Plan has tuning type, this means it has SM-specific tuning
-      // so loop through sm_list to find match, 
+      // so loop through sm_list to find match,
       // otherwise just specialize on provided SM
       typedef thrust::detail::conditional<has_tuning_t<Plan<lowest_supported_sm_arch> >::value,
                                   specialize_plan_impl_loop<Plan, SM, sm_list>,
                                   Plan<SM> >
           type;
     };
-    
+
     template <template <class> class Plan, class SM = THRUST_TUNING_ARCH>
     struct specialize_plan : specialize_plan_msvc10_war<Plan,SM>::type::type {};
 
@@ -342,14 +354,30 @@ namespace core {
     typename get_plan<Agent>::type THRUST_RUNTIME_FUNCTION
     get_agent_plan(int ptx_version)
     {
-#if (CUB_PTX_ARCH > 0) && defined(__THRUST_HAS_CUDART__)
-      typedef typename get_plan<Agent>::type Plan;
-      THRUST_UNUSED_VAR(ptx_version);
-      // We're on device, use default policy
-      return Plan(typename Agent::ptx_plan());
-#else
-      return get_agent_plan_impl<Agent, sm_list>::get(ptx_version);
-#endif
+      // Use one path, with Agent::ptx_plan, for device code where device-side
+      // kernel launches are supported. The other path, with
+      // get_agent_plan_impl::get(version), is for host code and for device
+      // code without device-side kernel launches. NVCC and Feta check for
+      // these situations differently.
+      #ifdef __NVCOMPILER_CUDA__
+        #ifdef __THRUST_HAS_CUDART__
+          if (CUB_IS_DEVICE_CODE) {
+            return typename get_plan<Agent>::type(typename Agent::ptx_plan());
+          } else
+        #endif
+        {
+          return get_agent_plan_impl<Agent, sm_list>::get(ptx_version);
+        }
+      #else
+        #if (CUB_PTX_ARCH > 0) && defined(__THRUST_HAS_CUDART__)
+          typedef typename get_plan<Agent>::type Plan;
+          THRUST_UNUSED_VAR(ptx_version);
+          // We're on device, use default policy
+          return Plan(typename Agent::ptx_plan());
+        #else
+          return get_agent_plan_impl<Agent, sm_list>::get(ptx_version);
+        #endif
+      #endif
     }
 
 // XXX keep this dead-code for now as a gentle reminder
@@ -433,67 +461,12 @@ namespace core {
   /////////////////////////
   /////////////////////////
 
-  inline cudaError_t CUB_RUNTIME_FUNCTION
-  get_occ_device_properties(cudaOccDeviceProp &occ_prop, int dev_id)
-  {
-    cudaError_t status = cudaSuccess;
-#ifdef __CUDA_ARCH__
-    {
-      cudaOccDeviceProp &o = occ_prop;
-      //
-      status = cudaDeviceGetAttribute(&o.computeMajor,
-                                      cudaDevAttrComputeCapabilityMajor,
-                                      dev_id);
-      status = cudaDeviceGetAttribute(&o.computeMinor,
-                                      cudaDevAttrComputeCapabilityMinor,
-                                      dev_id);
-      status = cudaDeviceGetAttribute(&o.maxThreadsPerBlock,
-                                      cudaDevAttrMaxThreadsPerBlock,
-                                      dev_id);
-      status = cudaDeviceGetAttribute(&o.maxThreadsPerMultiprocessor,
-                                      cudaDevAttrMaxThreadsPerMultiProcessor,
-                                      dev_id);
-      status = cudaDeviceGetAttribute(&o.regsPerBlock,
-                                      cudaDevAttrMaxRegistersPerBlock,
-                                      dev_id);
-      status = cudaDeviceGetAttribute(&o.regsPerMultiprocessor,
-                                      cudaDevAttrMaxRegistersPerMultiprocessor,
-                                      dev_id);
-      status = cudaDeviceGetAttribute(&o.warpSize,
-                                      cudaDevAttrWarpSize,
-                                      dev_id);
-
-      int i32value;
-      status = cudaDeviceGetAttribute(&i32value,
-                                      cudaDevAttrMaxSharedMemoryPerBlock,
-                                      dev_id);
-      o.sharedMemPerBlock = static_cast<size_t>(i32value);
-
-      status = cudaDeviceGetAttribute(&i32value,
-                                      cudaDevAttrMaxSharedMemoryPerMultiprocessor,
-                                      dev_id);
-      o.sharedMemPerMultiprocessor = static_cast<size_t>(i32value);
-
-      status = cudaDeviceGetAttribute(&o.numSms,
-                                      cudaDevAttrMultiProcessorCount,
-                                      dev_id);
-    }
-#else
-    {
-      cudaDeviceProp props;
-      status   = cudaGetDeviceProperties(&props, dev_id);
-      occ_prop = cudaOccDeviceProp(props);
-    }
-#endif
-    return status;
-  }
-  
-  int CUB_RUNTIME_FUNCTION
-  inline get_sm_count()
+  THRUST_RUNTIME_FUNCTION
+  int get_sm_count()
   {
     int dev_id;
     cuda_cub::throw_on_error(cudaGetDevice(&dev_id),
-                             "get_sm_count:"
+                             "get_sm_count :"
                              "failed to cudaGetDevice");
 
     cudaError_t status;
@@ -507,8 +480,8 @@ namespace core {
     return i32value;
   }
 
-  size_t CUB_RUNTIME_FUNCTION
-  inline get_max_shared_memory_per_block()
+  size_t THRUST_RUNTIME_FUNCTION
+  get_max_shared_memory_per_block()
   {
     int dev_id;
     cuda_cub::throw_on_error(cudaGetDevice(&dev_id),
@@ -527,8 +500,8 @@ namespace core {
     return static_cast<size_t>(i32value);
   }
 
-  size_t CUB_RUNTIME_FUNCTION
-  inline virtual_shmem_size(size_t shmem_per_block)
+  size_t THRUST_RUNTIME_FUNCTION
+  virtual_shmem_size(size_t shmem_per_block)
   {
     size_t max_shmem_per_block = core::get_max_shared_memory_per_block();
     if (shmem_per_block > max_shmem_per_block)
@@ -536,9 +509,9 @@ namespace core {
     else
       return 0;
   }
-  
-  size_t CUB_RUNTIME_FUNCTION
-  inline vshmem_size(size_t shmem_per_block, size_t num_blocks)
+
+  size_t THRUST_RUNTIME_FUNCTION
+  vshmem_size(size_t shmem_per_block, size_t num_blocks)
   {
     size_t max_shmem_per_block = core::get_max_shared_memory_per_block();
     if (shmem_per_block > max_shmem_per_block)
@@ -547,51 +520,6 @@ namespace core {
       return 0;
   }
 
-  template <class Kernel>
-  int CUB_RUNTIME_FUNCTION 
-  get_max_block_size(Kernel k)
-  {
-    int devId;
-    cuda_cub::throw_on_error(cudaGetDevice(&devId),
-                   "get_max_block_size :"
-                   "failed to cudaGetDevice");
-
-    cudaOccDeviceProp occ_prop;
-    cuda_cub::throw_on_error(get_occ_device_properties(occ_prop, devId),
-                   "get_max_block_size: "
-                   "failed to cudaGetDeviceProperties");
-
-
-    cudaFuncAttributes attribs;
-    cuda_cub::throw_on_error(cudaFuncGetAttributes(&attribs, reinterpret_cast<void *>(k)),
-                   "get_max_block_size: "
-                   "failed to cudaFuncGetAttributes");
-    cudaOccFuncAttributes occ_attrib(attribs);
-
-
-    cudaFuncCache cacheConfig;
-    cuda_cub::throw_on_error(cudaDeviceGetCacheConfig(&cacheConfig),
-                   "get_max_block_size: "
-                   "failed to cudaDeviceGetCacheConfig");
-
-    cudaOccDeviceState occ_state;
-    occ_state.cacheConfig      = (cudaOccCacheConfig)cacheConfig;
-    int          block_size    = 0;
-    int          min_grid_size = 0;
-    cudaOccError occ_status    = cudaOccMaxPotentialOccupancyBlockSize(&min_grid_size,
-                                                                    &block_size,
-                                                                    &occ_prop,
-                                                                    &occ_attrib,
-                                                                    &occ_state,
-                                                                    0);
-    if (CUDA_OCC_SUCCESS != occ_status || block_size <= 0)
-      cuda_cub::throw_on_error(cudaErrorInvalidConfiguration,
-                     "get_max_block_size: "
-                     "failed to cudaOccMaxPotentialOccupancyBlockSize");
-
-    return block_size;
-  }
-  
   // LoadIterator
   // ------------
   // if trivial iterator is passed, wrap loads into LDG
@@ -616,7 +544,7 @@ namespace core {
   {
     return raw_pointer_cast(&*it);
   }
-  
+
   template <class PtxPlan, class It>
   typename LoadIterator<PtxPlan, It>::type __device__ __forceinline__
   make_load_iterator_impl(It it, thrust::detail::false_type /* is_trivial */)
@@ -657,7 +585,7 @@ namespace core {
 
         type;
   };
-  
+
   // BlockStore
   // -----------
   // a helper metaprogram that returns type of a block loader
@@ -704,7 +632,7 @@ namespace core {
     __host__ __device__ operator T const &() const { return value_; }
   };
 
-  inline cuda_optional<int> CUB_RUNTIME_FUNCTION
+  cuda_optional<int> THRUST_RUNTIME_FUNCTION
   get_ptx_version()
   {
     int ptx_version = 0;
@@ -712,7 +640,7 @@ namespace core {
     return cuda_optional<int>(ptx_version, status);
   }
 
-  inline cudaError_t CUB_RUNTIME_FUNCTION
+  cudaError_t THRUST_RUNTIME_FUNCTION
   sync_stream(cudaStream_t stream)
   {
     return cub::SyncStream(stream);
@@ -724,7 +652,10 @@ namespace core {
   }
 
 #define CUDA_CUB_RET_IF_FAIL(e) \
-  if (thrust::cuda_cub::cub::Debug((e), __FILE__, __LINE__)) return e;
+  {                             \
+    auto const error = (e);     \
+    if (cub::Debug(error, __FILE__, __LINE__)) return error; \
+  }
 
   // uninitialized
   // -------
@@ -749,7 +680,7 @@ namespace core {
 
     __host__ __device__ __forceinline__ operator T&() { return get(); }
   };
-  
+
   // uninitialized_array
   // --------------
   // allocates uninitialized data on stack
@@ -837,6 +768,6 @@ using core::sm60;
 using core::sm52;
 using core::sm35;
 using core::sm30;
-} // namespace cuda_ 
+} // namespace cuda_
 
-THRUST_END_NS
+} // end namespace thrust
