@@ -36,7 +36,7 @@
 #include <thrust/system/cuda/detail/util.h>
 #include <thrust/detail/raw_reference_cast.h>
 #include <thrust/detail/type_traits/iterator/is_output_iterator.h>
-#include <thrust/system/cuda/detail/cub/device/device_reduce.cuh>
+#include <cub/device/device_reduce.cuh>
 #include <thrust/system/cuda/detail/par_to_seq.h>
 #include <thrust/system/cuda/detail/core/agent_launcher.h>
 #include <thrust/system/cuda/detail/get_value.h>
@@ -47,7 +47,8 @@
 #include <thrust/distance.h>
 #include <thrust/detail/alignment.h>
 
-THRUST_BEGIN_NS
+namespace thrust
+{
 
 template <typename DerivedPolicy,
           typename InputIterator1,
@@ -68,7 +69,7 @@ reduce_by_key(
 namespace cuda_cub {
 
 namespace __reduce_by_key {
-  
+
   template<bool> struct is_true : thrust::detail::false_type {};
   template<> struct is_true<true> : thrust::detail::true_type {};
 
@@ -78,16 +79,14 @@ namespace __reduce_by_key {
             int                     _ITEMS_PER_THREAD = 1,
             cub::BlockLoadAlgorithm _LOAD_ALGORITHM   = cub::BLOCK_LOAD_DIRECT,
             cub::CacheLoadModifier  _LOAD_MODIFIER    = cub::LOAD_DEFAULT,
-            cub::BlockScanAlgorithm _SCAN_ALGORITHM   = cub::BLOCK_SCAN_WARP_SCANS,
-            int                     _MIN_BLOCKS       = 1>
+            cub::BlockScanAlgorithm _SCAN_ALGORITHM   = cub::BLOCK_SCAN_WARP_SCANS>
   struct PtxPolicy
   {
     enum
     {
       BLOCK_THREADS    = _BLOCK_THREADS,
       ITEMS_PER_THREAD = _ITEMS_PER_THREAD,
-      ITEMS_PER_TILE   = BLOCK_THREADS * ITEMS_PER_THREAD,
-      MIN_BLOCKS       = _MIN_BLOCKS
+      ITEMS_PER_TILE   = BLOCK_THREADS * ITEMS_PER_THREAD
     };
 
     static const cub::BlockLoadAlgorithm LOAD_ALGORITHM = _LOAD_ALGORITHM;
@@ -97,7 +96,7 @@ namespace __reduce_by_key {
 
   template <class Arch, class Key, class Value>
   struct Tuning;
-  
+
   template <class Key, class Value>
   struct Tuning<sm30, Key, Value>
   {
@@ -146,7 +145,7 @@ namespace __reduce_by_key {
                         ((NOMINAL_4B_ITEMS_PER_THREAD * 8) +
                          Tuning::COMBINED_INPUT_BYTES - 1) /
                             Tuning::COMBINED_INPUT_BYTES>::value>::value,
-    };  
+    };
 
     typedef PtxPolicy<128,
                       ITEMS_PER_THREAD,
@@ -155,7 +154,7 @@ namespace __reduce_by_key {
                       cub::BLOCK_SCAN_WARP_SCANS>
         type;
   };    // Tuning sm35
-  
+
   template<class Key, class Value>
   struct Tuning<sm52,Key,Value> : Tuning<sm30,Key,Value>
   {
@@ -175,7 +174,7 @@ namespace __reduce_by_key {
                         ((NOMINAL_4B_ITEMS_PER_THREAD * 8) +
                          Tuning::COMBINED_INPUT_BYTES - 1) /
                             Tuning::COMBINED_INPUT_BYTES>::value>::value,
-    };  
+    };
 
     typedef PtxPolicy<256,
                       ITEMS_PER_THREAD,
@@ -400,7 +399,7 @@ namespace __reduce_by_key {
       //---------------------------------------------------------------------
       // Scatter utility methods
       //---------------------------------------------------------------------
-    
+
       // Directly scatter flagged items to output offsets
       // (specialized for IS_SEGMENTED_REDUCTION_FIXUP == false)
       THRUST_DEVICE_FUNCTION void scatter_direct(
@@ -424,7 +423,7 @@ namespace __reduce_by_key {
       // (specialized for IS_SEGMENTED_REDUCTION_FIXUP == false
       //
       // The exclusive scan causes each head flag to be paired with
-      // the previous value aggregate: 
+      // the previous value aggregate:
       //   * the scatter offsets must be decremented for value aggregates
       //
       THRUST_DEVICE_FUNCTION void scatter_two_phase(
@@ -503,7 +502,7 @@ namespace __reduce_by_key {
         // Last thread will output final count and last item, if necessary
         if (threadIdx.x == BLOCK_THREADS - 1)
         {
-          // If the last tile is a whole tile, the inclusive prefix 
+          // If the last tile is a whole tile, the inclusive prefix
           // contains accumulated value reduction for the last segment
           if (num_remaining == ITEMS_PER_TILE)
           {
@@ -517,7 +516,7 @@ namespace __reduce_by_key {
           *num_runs_output_it = num_segments;
         }
       }
-    
+
       //---------------------------------------------------------------------
       // Cooperatively scan a device-wide sequence of tiles with other CTAs
       //---------------------------------------------------------------------
@@ -605,7 +604,7 @@ namespace __reduce_by_key {
           if (!IS_LAST_TILE)
             tile_state.SetInclusive(0, tile_aggregate);
 
-          // Initialize the segment index for the first scan item if necessary 
+          // Initialize the segment index for the first scan item if necessary
           // (the exclusive prefix for the first item is garbage)
           if (!HAS_IDENTITY_ZERO)
             scan_items[0].key = 0;
@@ -930,7 +929,7 @@ namespace __reduce_by_key {
     {
       return status;
     }
-    
+
     ScanTileState tile_state;
     status = tile_state.Init(static_cast<int>(num_tiles), allocations[0], allocation_sizes[0]);
     CUDA_CUB_RET_IF_FAIL(status);
@@ -985,7 +984,7 @@ namespace __reduce_by_key {
     size_t       temp_storage_bytes = 0;
     cudaStream_t stream             = cuda_cub::stream(policy);
     bool         debug_sync         = THRUST_DEBUG_SYNC_FLAG;
-    
+
     if (num_items == 0)
       return thrust::make_pair(keys_output, values_output);
 
@@ -1059,7 +1058,7 @@ namespace __reduce_by_key {
 // Thrust API entry points
 //-------------------------
 
-__thrust_exec_check_disable__ 
+__thrust_exec_check_disable__
 template <class Derived,
           class KeyInputIt,
           class ValInputIt,
@@ -1161,7 +1160,7 @@ reduce_by_key(execution_policy<Derived> &policy,
 
 } // namespace cuda_
 
-THRUST_END_NS
+} // end namespace thrust
 
 #include <thrust/memory.h>
 #include <thrust/reduce.h>
