@@ -42,7 +42,8 @@
 #include <thrust/distance.h>
 #include <thrust/detail/alignment.h>
 
-THRUST_BEGIN_NS
+namespace thrust
+{
 
 namespace cuda_cub {
 
@@ -50,35 +51,36 @@ namespace __set_operations {
 
   template <bool UpperBound,
             class IntT,
+            class Size,
             class It,
             class T,
             class Comp>
   THRUST_DEVICE_FUNCTION void
   binary_search_iteration(It   data,
-                          int &begin,
-                          int &end,
+                          Size &begin,
+                          Size &end,
                           T    key,
                           int  shift,
                           Comp comp)
   {
 
     IntT scale = (1 << shift) - 1;
-    int  mid   = (int)((begin + scale * end) >> shift);
+    Size mid   = (begin + scale * end) >> shift;
 
     T    key2 = data[mid];
     bool pred = UpperBound ? !comp(key, key2) : comp(key2, key);
     if (pred)
-      begin = (int)mid + 1;
+      begin = mid + 1;
     else
       end = mid;
   }
 
-  template <bool UpperBound, class T, class It, class Comp>
-  THRUST_DEVICE_FUNCTION int
-  binary_search(It data, int count, T key, Comp comp)
+  template <bool UpperBound, class Size, class T, class It, class Comp>
+  THRUST_DEVICE_FUNCTION Size
+  binary_search(It data, Size count, T key, Comp comp)
   {
-    int begin = 0;
-    int end   = count;
+    Size begin = 0;
+    Size end   = count;
     while (begin < end)
       binary_search_iteration<UpperBound, int>(data,
                                                begin,
@@ -89,12 +91,12 @@ namespace __set_operations {
     return begin;
   }
 
-  template <bool UpperBound, class IntT, class T, class It, class Comp>
-  THRUST_DEVICE_FUNCTION int
-  biased_binary_search(It data, int count, T key, IntT levels, Comp comp)
+  template <bool UpperBound, class IntT, class Size, class T, class It, class Comp>
+  THRUST_DEVICE_FUNCTION Size
+  biased_binary_search(It data, Size count, T key, IntT levels, Comp comp)
   {
-    int begin = 0;
-    int end   = count;
+    Size begin = 0;
+    Size end   = count;
 
     if (levels >= 4 && begin < end)
       binary_search_iteration<UpperBound, IntT>(data, begin, end, key, 9, comp);
@@ -110,18 +112,18 @@ namespace __set_operations {
     return begin;
   }
 
-  template <bool UpperBound, class It1, class It2, class Comp>
-  THRUST_DEVICE_FUNCTION int
-  merge_path(It1 a, int aCount, It2 b, int bCount, int diag, Comp comp)
+  template <bool UpperBound, class Size, class It1, class It2, class Comp>
+  THRUST_DEVICE_FUNCTION Size
+  merge_path(It1 a, Size aCount, It2 b, Size bCount, Size diag, Comp comp)
   {
     typedef typename thrust::iterator_traits<It1>::value_type T;
 
-    int begin = thrust::max(0, diag - bCount);
-    int end   = thrust::min(diag, aCount);
+    Size begin = thrust::max<Size>(0, diag - bCount);
+    Size end   = thrust::min<Size>(diag, aCount);
 
     while (begin < end)
     {
-      int  mid  = (begin + end) >> 1;
+      Size  mid  = (begin + end) >> 1;
       T    aKey = a[mid];
       T    bKey = b[diag - 1 - mid];
       bool pred = UpperBound ? comp(aKey, bKey) : !comp(bKey, aKey);
@@ -132,9 +134,9 @@ namespace __set_operations {
     }
     return begin;
   }
-  
+
   template <class It1, class It2, class Size, class Size2, class CompareOp>
-  pair<Size, Size> THRUST_DEVICE_FUNCTION
+  THRUST_DEVICE_FUNCTION pair<Size, Size>
   balanced_path(It1       keys1,
                 It2       keys2,
                 Size      num_keys1,
@@ -202,15 +204,13 @@ namespace __set_operations {
             int                      _ITEMS_PER_THREAD = 1,
             cub::BlockLoadAlgorithm  _LOAD_ALGORITHM   = cub::BLOCK_LOAD_DIRECT,
             cub::CacheLoadModifier   _LOAD_MODIFIER    = cub::LOAD_LDG,
-            cub::BlockScanAlgorithm  _SCAN_ALGORITHM   = cub::BLOCK_SCAN_WARP_SCANS,
-            int                      _MIN_BLOCKS       = 1>
+            cub::BlockScanAlgorithm  _SCAN_ALGORITHM   = cub::BLOCK_SCAN_WARP_SCANS>
   struct PtxPolicy
   {
     enum
     {
       BLOCK_THREADS    = _BLOCK_THREADS,
       ITEMS_PER_THREAD = _ITEMS_PER_THREAD,
-      MIN_BLOCKS       = _MIN_BLOCKS,
       ITEMS_PER_TILE   = _BLOCK_THREADS * _ITEMS_PER_THREAD - 1
     };
 
@@ -221,9 +221,9 @@ namespace __set_operations {
 
   template<class Arch, class T, class U>
   struct Tuning;
-  
+
   namespace mpl = thrust::detail::mpl::math;
-  
+
   template<class T, class U>
   struct Tuning<sm30,T,U>
   {
@@ -324,9 +324,9 @@ namespace __set_operations {
 
     typedef key1_type  key_type;
     typedef value1_type value_type;
-    
+
     typedef cub::ScanTileState<Size> ScanTileState;
-    
+
     template <class Arch>
     struct PtxPlan : Tuning<Arch, key_type, value_type>::type
     {
@@ -436,7 +436,7 @@ namespace __set_operations {
       CompareOp      compare_op;
       SetOp          set_op;
       pair<Size, Size> *partitions;
-      Size *output_count;
+      std::size_t *output_count;
 
       //---------------------------------------------------------------------
       // Utility functions
@@ -498,7 +498,7 @@ namespace __set_operations {
           output[idx] = input[ITEM];
         }
       }
-      
+
       template <class OutputIt, class T, class SharedIt>
       void THRUST_DEVICE_FUNCTION
       scatter(OutputIt output,
@@ -510,7 +510,7 @@ namespace __set_operations {
               int      tile_output_count)
       {
         using core::sync_threadblock;
-        
+
 
 
         int local_scatter_idx = thread_output_prefix - tile_output_prefix;
@@ -578,9 +578,9 @@ namespace __set_operations {
         //
         int num_keys1 = static_cast<int>(keys1_end - keys1_beg);
         int num_keys2 = static_cast<int>(keys2_end - keys2_beg);
-        
-       
-       // load keys into shared memory for further processing 
+
+
+       // load keys into shared memory for further processing
         key_type keys_loc[ITEMS_PER_THREAD];
 
         gmem_to_reg<!IS_LAST_TILE>(keys_loc,
@@ -588,7 +588,7 @@ namespace __set_operations {
                                    keys2_in + keys2_beg,
                                    num_keys1,
                                    num_keys2);
-        
+
         reg_to_shared(&storage.keys_shared[0], keys_loc);
 
         sync_threadblock();
@@ -604,7 +604,7 @@ namespace __set_operations {
                           diag_loc,
                           4,
                           compare_op);
-        
+
         int keys1_beg_loc = partition_loc.first;
         int keys2_beg_loc = partition_loc.second;
 
@@ -628,7 +628,7 @@ namespace __set_operations {
 
         int num_keys1_loc = keys1_end_loc - keys1_beg_loc;
         int num_keys2_loc = keys2_end_loc - keys2_beg_loc;
-        
+
         // perform serial set operation
         //
         int indices[ITEMS_PER_THREAD];
@@ -758,7 +758,7 @@ namespace __set_operations {
            CompareOp      compare_op_,
            SetOp          set_op_,
            pair<Size, Size> *partitions_,
-           Size *output_count_)
+           std::size_t * output_count_)
           : storage(storage_),
             tile_state(tile_state_),
             keys1_in(core::make_load_iterator(ptx_plan(), keys1_)),
@@ -772,7 +772,7 @@ namespace __set_operations {
             compare_op(compare_op_),
             set_op(set_op_),
             partitions(partitions_),
-            output_count(output_count_) 
+            output_count(output_count_)
       {
         int  tile_idx      = blockIdx.x;
         int  num_tiles     = gridDim.x;
@@ -781,7 +781,7 @@ namespace __set_operations {
         {
           consume_tile<false>(tile_idx);
         }
-        else 
+        else
         {
           consume_tile<true>(tile_idx);
         }
@@ -803,7 +803,7 @@ namespace __set_operations {
                        CompareOp      compare_op,
                        SetOp          set_op,
                        pair<Size, Size> *partitions,
-                       Size *        output_count,
+                       std::size_t *  output_count,
                        ScanTileState tile_state,
                        char *        shmem)
     {
@@ -825,7 +825,7 @@ namespace __set_operations {
            output_count);
     }
   };    // struct SetOpAgent
-  
+
   template <class KeysIt1,
             class KeysIt2,
             class Size,
@@ -867,7 +867,7 @@ namespace __set_operations {
       }
     }
   };    // struct PartitionAgent
-  
+
   template <class ScanTileState,
             class Size>
   struct InitAgent
@@ -939,7 +939,7 @@ namespace __set_operations {
       return active_mask;
     }
   };    // struct serial_set_intersection
-  
+
   // serial_set_symmetric_difference
   // ---------------------
   // emit A if A < B and emit B if B < A.
@@ -984,8 +984,8 @@ namespace __set_operations {
         // The outputs must come from A by definition of set difference.
         output[i]  = pA ? aKey : bKey;
         indices[i] = pA ? aBegin : bBegin;
-        
-        if (aBegin + bBegin < end && pA != pB) 
+
+        if (aBegin + bBegin < end && pA != pB)
           active_mask |= 1 << i;
 
         if (!pB) {aKey = keys[++aBegin]; }
@@ -1039,7 +1039,7 @@ namespace __set_operations {
         // The outputs must come from A by definition of set difference.
         output[i]  = aKey;
         indices[i] = aBegin;
-        
+
         if (aBegin + bBegin < end && pA)
           active_mask |= 1 << i;
 
@@ -1049,7 +1049,7 @@ namespace __set_operations {
       return active_mask;
     }
   };    // struct set_difference
-  
+
   // serial_set_union
   // ----------------
   // emit A if A <= B else emit B
@@ -1093,7 +1093,7 @@ namespace __set_operations {
         // Output A in case of a tie, so check if b < a.
         output[i]  = pB ? bKey : aKey;
         indices[i] = pB ? bBegin : aBegin;
-        
+
         if (aBegin + bBegin < end)
           active_mask |= 1 << i;
 
@@ -1126,7 +1126,7 @@ namespace __set_operations {
             Size           num_keys2,
             KeysOutputIt   keys_output,
             ValuesOutputIt values_output,
-            Size *         output_count,
+            std::size_t *  output_count,
             CompareOp      compare_op,
             SetOp          set_op,
             cudaStream_t   stream,
@@ -1137,7 +1137,7 @@ namespace __set_operations {
       return cudaErrorNotSupported;
 
     cudaError_t status = cudaSuccess;
-    
+
     using core::AgentPlan;
     using core::AgentLauncher;
 
@@ -1156,7 +1156,7 @@ namespace __set_operations {
 
     typedef AgentLauncher<PartitionAgent<KeysIt1, KeysIt2, Size, CompareOp> >
         partition_agent;
-    
+
     typedef typename set_op_agent::ScanTileState ScanTileState;
     typedef AgentLauncher<InitAgent<ScanTileState, Size> > init_agent;
 
@@ -1169,7 +1169,7 @@ namespace __set_operations {
     Size num_tiles = (keys_total + tile_size - 1) / tile_size;
 
     size_t tile_agent_storage;
-    status = ScanTileState::AllocationSize(static_cast<int>(num_tiles), tile_agent_storage);
+    status = ScanTileState::AllocationSize(num_tiles, tile_agent_storage);
     CUDA_CUB_RET_IF_FAIL(status);
 
     size_t vshmem_storage = core::vshmem_size(set_op_plan.shared_memory_size,
@@ -1193,7 +1193,7 @@ namespace __set_operations {
     }
 
     ScanTileState tile_state;
-    status = tile_state.Init(static_cast<int>(num_tiles), allocations[0], allocation_sizes[0]);
+    status = tile_state.Init(num_tiles, allocations[0], allocation_sizes[0]);
     CUDA_CUB_RET_IF_FAIL(status);
 
     pair<Size, Size> *partitions = (pair<Size, Size> *)allocations[1];
@@ -1264,30 +1264,31 @@ namespace __set_operations {
 
     if (num_keys1 + num_keys2 == 0)
       return thrust::make_pair(keys_output, values_output);
-     
+
     size_t       temp_storage_bytes = 0;
     cudaStream_t stream             = cuda_cub::stream(policy);
     bool         debug_sync         = THRUST_DEBUG_SYNC_FLAG;
 
     cudaError_t status;
-    status = doit_step<HAS_VALUES>(NULL,
+    THRUST_DOUBLE_INDEX_TYPE_DISPATCH(status, doit_step<HAS_VALUES>,
+        num_keys1, num_keys2, (NULL,
                                    temp_storage_bytes,
                                    keys1_first,
                                    keys2_first,
                                    values1_first,
                                    values2_first,
-                                   num_keys1,
-                                   num_keys2,
+                                   num_keys1_fixed,
+                                   num_keys2_fixed,
                                    keys_output,
                                    values_output,
-                                   reinterpret_cast<size_type*>(NULL),
+                                   reinterpret_cast<std::size_t*>(NULL),
                                    compare_op,
                                    set_op,
                                    stream,
-                                   debug_sync);
+                                   debug_sync));
     cuda_cub::throw_on_error(status, "set_operations failed on 1st step");
 
-    size_t allocation_sizes[2] = {sizeof(size_type), temp_storage_bytes};
+    size_t allocation_sizes[2] = {sizeof(std::size_t), temp_storage_bytes};
     void * allocations[2]      = {NULL, NULL};
 
     size_t storage_size = 0;
@@ -1309,30 +1310,31 @@ namespace __set_operations {
                                  allocation_sizes);
     cuda_cub::throw_on_error(status, "set_operations failed on 2nd alias_storage");
 
-    size_type* d_output_count
-      = thrust::detail::aligned_reinterpret_cast<size_type*>(allocations[0]);
+    std::size_t* d_output_count
+      = thrust::detail::aligned_reinterpret_cast<std::size_t*>(allocations[0]);
 
-    status = doit_step<HAS_VALUES>(allocations[1],
+    THRUST_DOUBLE_INDEX_TYPE_DISPATCH(status, doit_step<HAS_VALUES>,
+        num_keys1, num_keys2, (allocations[1],
                                    temp_storage_bytes,
                                    keys1_first,
                                    keys2_first,
                                    values1_first,
                                    values2_first,
-                                   num_keys1,
-                                   num_keys2,
+                                   num_keys1_fixed,
+                                   num_keys2_fixed,
                                    keys_output,
                                    values_output,
                                    d_output_count,
                                    compare_op,
                                    set_op,
                                    stream,
-                                   debug_sync);
+                                   debug_sync));
     cuda_cub::throw_on_error(status, "set_operations failed on 2nd step");
-    
+
     status = cuda_cub::synchronize(policy);
     cuda_cub::throw_on_error(status, "set_operations failed to synchronize");
 
-    size_type output_count = cuda_cub::get_value(policy, d_output_count);
+    std::size_t output_count = cuda_cub::get_value(policy, d_output_count);
 
     return thrust::make_pair(keys_output + output_count, values_output + output_count);
   }
@@ -1992,5 +1994,5 @@ set_union_by_key(execution_policy<Derived> &policy,
 }
 
 }    // namespace cuda_cub
-THRUST_END_NS
+} // end namespace thrust
 #endif
