@@ -35,7 +35,7 @@ struct par_t : thrust::system::tbb::detail::execution_policy<par_t>,
     thrust::system::tbb::detail::execution_policy>
 {
   __host__ __device__
-  par_t() : thrust::system::tbb::detail::execution_policy<par_t>() {}
+  THRUST_CONSTEXPR par_t() : thrust::system::tbb::detail::execution_policy<par_t>() {}
 };
 
 
